from django.contrib import admin

from django.utils.translation import ugettext_lazy as _

from ..utils import get_image_tag


class SponsorAdmin(admin.ModelAdmin):
    """
    Admin interface that is used to edit sponsor objects in the CMS.
    """

    fields = [
        "name",
        "url",
        "logo",
        "image_tag",
    ]
    readonly_fields = ["image_tag"]
    list_display = [
        "name",
        "has_logo",
    ]
    list_per_page = 25

    def has_logo(self, obj):
        """
        Additional field to display whether a logo is set for the sponsor in the list view.

        :return: Whether the sponsor has a logo
        :rtype: bool
        """

        return bool(obj.logo)

    has_logo.boolean = True
    has_logo.short_description = _("logo")

    def image_tag(self, obj):
        """
        Image thumbnail to display a preview of a image in the form view.

        :return: img HTML tag to display an image thumbnail
        :rtype: str
        """
        return get_image_tag(obj.logo)

    image_tag.short_description = ""

    class Media:
        js = ("js/image_preview.js",)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from .static import upload_sponsor_logos
from ..validators import validate_multiple_extensions


class Sponsor(models.Model):
    """
    Model to manage our sponsors that are shown in the app.
    """

    name = models.CharField(
        verbose_name=_("name"),
        max_length=255,
        blank=False,
    )
    url = models.URLField(
        blank=True,
        verbose_name=_("URL"),
    )
    logo = models.ImageField(
        upload_to=upload_sponsor_logos,
        validators=[validate_multiple_extensions],
        blank=True,
        verbose_name=_("logo"),
    )

    def __str__(self):
        """
        String representation of sponsor name

        :return: name of the sponsor
        :rtype: str
        """
        return self.name

    class Meta:
        """
        Define user readable name of sponsors
        """

        verbose_name = _("sponsor")
        verbose_name_plural = _("sponsors")



# Generated by Django 3.2.16 on 2023-05-03 14:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cms", "0009_sponsor"),
    ]

    operations = [
        migrations.AddField(
            model_name="sponsor",
            name="url",
            field=models.URLField(blank=True, verbose_name="URL"),
        ),
    ]



from collections import defaultdict

from django.contrib import admin
from django.db.models import F
from django.utils.translation import ugettext_lazy as _
from .models import Discipline, TrainingSet


class DisciplineListFilter(admin.SimpleListFilter):
    """
    Generic Filter for models that have a direct relationship to disciplines.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("disciplines")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "disciplines"

    template = "admin/discipline_filter.html"

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable.

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """

        # Verify that only disciplines are displayed that actually can contain training sets
        queryset = Discipline.objects.filter(lft=F("rght") - 1)

        if "training set" in request.GET:
            queryset = queryset.filter(training_sets=request.GET["training set"])

        if request.user.is_superuser:
            queryset = queryset.filter(creator_is_admin=True)
        else:
            queryset = queryset.filter(created_by__in=request.user.groups.all())

        list_of_disciplines = [
            (
                str(discipline.id),
                f"{discipline.parent} \u2794 {discipline}",
            )
            for discipline in queryset
        ]
        return sorted(list_of_disciplines, key=lambda tp: tp[1])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(discipline=self.value()).distinct()
        return queryset


class DocumentDisciplineListFilter(DisciplineListFilter):
    """
    Filter for disciplines within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(training_sets__discipline=self.value()).distinct()
        return queryset


class DocumentTrainingSetListFilter(admin.SimpleListFilter):
    """
    Filter for training sets within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("training sets")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "training set"

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable.

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each training set
        :rtype: list
        """
        queryset = TrainingSet.objects.all()
        if "disciplines" in request.GET:
            queryset = queryset.filter(discipline=request.GET["disciplines"])
        if request.user.is_superuser:
            queryset = queryset.filter(creator_is_admin=True)
        else:
            queryset = queryset.filter(created_by__in=request.user.groups.all())
        list_of_trainingsets = [
            ((str(trainingset.id), trainingset.title)) for trainingset in queryset
        ]
        return sorted(list_of_trainingsets, key=lambda tp: tp[1])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(training_sets__id=self.value()).distinct()
        return queryset


NONE = "none"
PENDING = "pending"
APPROVED = "approved"
NO_APPROVED = "no-approved"


class ApprovedImageListFilter(admin.SimpleListFilter):
    """
    Filter for approved images within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("Images")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "images"

    default_value = None

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """
        return (
            (APPROVED, _("At least one approved image")),
            (PENDING, _("At least one pending image")),
            (NO_APPROVED, _("No approved images")),
            (NONE, _("No images")),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """

        if self.value():
            if self.value() == NONE:
                return queryset.filter(document_image__isnull=True).distinct()
            elif self.value() == PENDING:
                return queryset.filter(document_image__confirmed=False).distinct()
            elif self.value() == APPROVED:
                return queryset.filter(document_image__confirmed=True).distinct()
            elif self.value() == NO_APPROVED:
                return queryset.exclude(document_image__confirmed=True).distinct()
        return queryset


NONE = "none"
AT_LEAST_ONE = "at-least-one"
RELEASED = "released"
RELEASED_DISCIPLINE = "released-discipline"
UNRELEASED = "unreleased"


class AssignedListFilter(admin.SimpleListFilter):
    """
    Filter for documents that are either assigned or unassigned to at least one training set.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("Assignments")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "assigned"

    default_value = None

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """
        return (
            (NONE, _("Not assigned to any module")),
            (AT_LEAST_ONE, _("Assigned to at least one module")),
            (RELEASED, _("Assigned to released modules")),
            (
                RELEASED_DISCIPLINE,
                _("Assigned to released modules in released disciplines"),
            ),
            (UNRELEASED, _("Assigned to unreleased modules")),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """

        filters = {}
        if self.value():
            if "disciplines" in request.GET:
                filters["training_sets__discipline"] = request.GET["disciplines"]
            if "training set" in request.GET:
                filters["training_sets"] = request.GET["training set"]
            if self.value() == NONE:
                filters["training_sets__isnull"] = True
            elif self.value() == AT_LEAST_ONE:
                filters["training_sets__isnull"] = False
            elif self.value() == RELEASED:
                filters["training_sets__released"] = True
            elif self.value() == RELEASED_DISCIPLINE:
                filters["training_sets__released"] = True
                filters["training_sets__discipline__released"] = True
            elif self.value() == UNRELEASED:
                filters["training_sets__released"] = False
        return queryset.filter(**filters).distinct()



"""
Register models for Django's CRUD back end and
specify autocomplete_fields, search_fields and nested modules
"""
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Discipline, TrainingSet, Document, GroupAPIKey, Feedback, Sponsor
from .admins import DisciplineAdmin
from .admins import TrainingSetAdmin
from .admins import DocumentAdmin
from .admins import GroupAPIKeyAdmin
from .admins import FeedbackAdmin
from .admins import SponsorAdmin


def get_app_list(self, request):
    """
    Function that returns a sorted list of all the installed apps that have been
    registered in this site.

    :param self: A handle to the :class:`admin.AdminSite`
    :type self: class: `admin.AdminSite`
    :param request: current user request
    :type request: django.http.request

    :return: list of app dictionaries (e.g. containing models)
    :rtype: list
    """
    ordering = {
        _("disciplines").capitalize(): 1,
        _("training sets").capitalize(): 2,
        _("vocabulary").capitalize(): 3,
        _("api keys").capitalize(): 4,
        _("feedback").capitalize(): 5,
    }
    app_dict = self._build_app_dict(request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: x["name"].lower())

    # Sort the respective modules according the defined order
    for app in app_list:
        try:
            app["models"].sort(key=lambda x: ordering[x["name"]])
        except KeyError:
            pass
    return app_list


admin.AdminSite.get_app_list = get_app_list
admin.site.register(Discipline, DisciplineAdmin)
admin.site.register(TrainingSet, TrainingSetAdmin)
admin.site.register(Document, DocumentAdmin)
admin.site.register(GroupAPIKey, GroupAPIKeyAdmin)
admin.site.register(Feedback, FeedbackAdmin)
admin.site.register(Sponsor, SponsorAdmin)



# Generated by Django 3.2.16 on 2023-02-20 10:29

from django.db import migrations, models
import lunes_cms.cms.models.static
import lunes_cms.cms.validators


class Migration(migrations.Migration):
    dependencies = [
        ("cms", "0008_add_numerals_adverbs_and_pronouns_as_word_types"),
    ]

    operations = [
        migrations.CreateModel(
            name="Sponsor",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="name")),
                (
                    "logo",
                    models.ImageField(
                        blank=True,
                        upload_to=lunes_cms.cms.models.static.upload_sponsor_logos,
                        validators=[
                            lunes_cms.cms.validators.validate_multiple_extensions
                        ],
                        verbose_name="logo",
                    ),
                ),
            ],
            options={
                "verbose_name": "sponsor",
                "verbose_name_plural": "sponsors",
            },
        ),
    ]



from rest_framework import viewsets

from ....cms.models import Sponsor
from ..serializers import SponsorSerializer


class SponsorsViewSet(viewsets.ModelViewSet):
    """
    View to provide a queryset of all current sponsors in the app.
    """

    serializer_class = SponsorSerializer
    http_method_names = ["get"]

    def get_queryset(self):
        """
        Get the queryset of the current sponsors managed by the cms.

        :return: The queryset of disciplines
        :rtype: ~django.db.models.query.QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Sponsor.objects.none()
        return Sponsor.objects.all()



"""
This is a collection of admin filter tags and filters
"""
from collections import defaultdict
import logging

from django import template

logger = logging.getLogger(__name__)
register = template.Library()


@register.simple_tag
def group_discipline_filter_choices(choices):
    """
    Group the discipline options by their parent discipline

    :param choices: The flat choices
    :type choices: list [ dict ]

    :return: The grouped choices
    :rtype: dict
    """
    options = []
    optgroups = defaultdict(list)
    delimiter = " \u2794 "
    for choice in choices:
        if delimiter not in choice["display"]:
            options.append(choice)
        else:
            parent, child = choice["display"].split(delimiter)
            choice["display"] = child
            optgroups[parent].append(choice)
    return {
        "options": options,
        "optgroups": dict(optgroups),
    }



# Generated by Django 3.2.18 on 2023-07-16 11:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cms", "0010_sponsor_url"),
    ]

    operations = [
        migrations.AddField(
            model_name="document",
            name="example_sentence",
            field=models.TextField(blank=True, verbose_name="example sentence"),
        ),
    ]



from rest_framework import serializers

from ....cms.models import Sponsor


class SponsorSerializer(serializers.ModelSerializer):
    """
    Serializer for the Sponsor module.
    """

    class Meta:
        """
        Define model and the corresponding fields for the display in the cms.
        """

        model = Sponsor
        fields = (
            "id",
            "name",
            "url",
            "logo",
        )



# Generated by Django 3.2.16 on 2023-02-20 20:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cms", "0007_document_created_by_link"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="word_type",
            field=models.CharField(
                choices=[
                    ("Nomen", "Substantiv"),
                    ("Verb", "Verb"),
                    ("Adjektiv", "Adjektiv"),
                    ("Numeral", "Numeral"),
                    ("Pronomen", "Pronomen"),
                    ("Adverb", "Adverb"),
                ],
                default="",
                max_length=255,
                verbose_name="word type",
            ),
        ),
    ]
